@@ -14,20 +14,16 @@
 
 package load
 
-// Files in package are to a large extent based on Go files from the following
-// Go packages:
-//    - cmd/go/internal/load
-//    - go/build
-
 import (
 	"context"
 	"fmt"
+	"sort"
+	"strconv"
 
 	"cuelang.org/go/cue/ast"
 	"cuelang.org/go/cue/build"
 	"cuelang.org/go/internal/cueexperiment"
 	"cuelang.org/go/internal/filetypes"
-	"cuelang.org/go/internal/mod/modimports"
 	"cuelang.org/go/internal/mod/modpkgload"
 	"cuelang.org/go/internal/mod/modrequirements"
 	"cuelang.org/go/mod/module"
@@ -59,6 +55,9 @@ func Instances(args []string, c *Config) []*build.Instance {
 		return []*build.Instance{c.newErrInstance(err)}
 	}
 	c = newC
+	if len(args) == 0 {
+		args = []string{"."}
+	}
 
 	// TODO: This requires packages to be placed before files. At some point this
 	// could be relaxed.
@@ -67,29 +66,39 @@ func Instances(args []string, c *Config) []*build.Instance {
 	}
 	pkgArgs := args[:i]
 	otherArgs := args[i:]
-
-	// Pass all arguments that look like packages to loadPackages
-	// so that they'll be available when looking up the packages
-	// that are specified on the command line.
-	// Relative import paths create a package with an associated
-	// error but it turns out that's actually OK because the cue/load
-	// logic resolves such paths without consulting pkgs.
-	pkgs, err := loadPackages(ctx, c, pkgArgs)
+	otherFiles, err := filetypes.ParseArgs(otherArgs)
 	if err != nil {
 		return []*build.Instance{c.newErrInstance(err)}
 	}
+	for _, f := range otherFiles {
+		if err := setFileSource(c, f); err != nil {
+			return []*build.Instance{c.newErrInstance(err)}
+		}
+	}
+	synCache := newSyntaxCache(c)
 	tg := newTagger(c)
-	l := newLoader(c, tg, pkgs)
+	// Pass all arguments that look like packages to loadPackages
+	// so that they'll be available when looking up the packages
+	// that are specified on the command line.
+	expandedPaths, err := expandPackageArgs(c, pkgArgs, c.Package, tg)
+	if err != nil {
+		return []*build.Instance{c.newErrInstance(err)}
+	}
+	pkgs, err := loadPackages(ctx, c, synCache, expandedPaths, otherFiles)
+	if err != nil {
+		return []*build.Instance{c.newErrInstance(err)}
+	}
+	l := newLoader(c, tg, synCache, pkgs)
 
 	if c.Context == nil {
 		c.Context = build.NewContext(
-			build.Loader(l.buildLoadFunc()),
+			build.Loader(l.loadFunc),
 			build.ParseFile(c.ParseFile),
 		)
 	}
 
 	a := []*build.Instance{}
-	if len(args) == 0 || i > 0 {
+	if len(pkgArgs) > 0 {
 		for _, m := range l.importPaths(pkgArgs) {
 			if m.Err != nil {
 				inst := c.newErrInstance(m.Err)
@@ -100,12 +109,8 @@ func Instances(args []string, c *Config) []*build.Instance {
 		}
 	}
 
-	if len(otherArgs) > 0 {
-		files, err := filetypes.ParseArgs(otherArgs)
-		if err != nil {
-			return []*build.Instance{c.newErrInstance(err)}
-		}
-		a = append(a, l.cueFilesPackage(files))
+	if len(otherFiles) > 0 {
+		a = append(a, l.cueFilesPackage(otherFiles))
 	}
 
 	for _, p := range a {
@@ -144,7 +149,9 @@ func Instances(args []string, c *Config) []*build.Instance {
 	return a
 }
 
-func loadPackages(ctx context.Context, cfg *Config, extraPkgs []string) (*modpkgload.Packages, error) {
+// loadPackages returns packages loaded from the given package list and also
+// including imports from the given build files.
+func loadPackages(ctx context.Context, cfg *Config, synCache *syntaxCache, pkgs []resolvedPackageArg, otherFiles []*build.File) (*modpkgload.Packages, error) {
 	if cfg.Registry == nil || cfg.modFile == nil || cfg.modFile.Module == "" {
 		return nil, nil
 	}
@@ -158,19 +165,46 @@ func loadPackages(ctx context.Context, cfg *Config, extraPkgs []string) (*modpkg
 		FS:  cfg.fileSystem.ioFS(cfg.ModuleRoot),
 		Dir: ".",
 	}
-	allImports, err := modimports.AllImports(modimports.AllModuleFiles(mainModLoc.FS, mainModLoc.Dir))
-	if err != nil {
-		return nil, fmt.Errorf("cannot enumerate all module imports: %v", err)
+	pkgPaths := make(map[string]bool)
+	// Add any packages specified directly on the command line.
+	for _, pkg := range pkgs {
+		pkgPaths[pkg.resolvedCanonical] = true
 	}
-	// Add any packages specified on the command line so they're always
-	// available.
-	allImports = append(allImports, extraPkgs...)
+	// Add any imports found in other files.
+	for _, f := range otherFiles {
+		if f.Encoding != build.CUE {
+			// not a CUE file; assume it has no imports for now.
+			continue
+		}
+		syntaxes, err := synCache.getSyntax(f)
+		if err != nil {
+			return nil, fmt.Errorf("cannot get syntax for %q: %v", f.Filename, err)
+		}
+		for _, syntax := range syntaxes {
+			for _, imp := range syntax.Imports {
+				pkgPath, err := strconv.Unquote(imp.Path.Value)
+				if err != nil {
+					// Should never happen.
+					return nil, fmt.Errorf("invalid import path %q in %s", imp.Path.Value, f.Filename)
+				}
+				// Canonicalize the path.
+				pkgPath = module.ParseImportPath(pkgPath).Canonical().String()
+				pkgPaths[pkgPath] = true
+			}
+		}
+	}
+	// TODO use maps.Keys when we can.
+	pkgPathSlice := make([]string, 0, len(pkgPaths))
+	for p := range pkgPaths {
+		pkgPathSlice = append(pkgPathSlice, p)
+	}
+	sort.Strings(pkgPathSlice)
 	return modpkgload.LoadPackages(
 		ctx,
 		cfg.Module,
 		mainModLoc,
 		reqs,
 		cfg.Registry,
-		allImports,
+		pkgPathSlice,
 	), nil
 }
