@@ -37,7 +37,7 @@ func main() {
 	}
 
 	for _, d := range dirs {
-		if !d.IsDir() {
+		if !d.IsDir() || d.Name() == "cue.mod" {
 			continue
 		}
 		dir := filepath.Join(cwd, "testdata")
@@ -45,7 +45,7 @@ func main() {
 		inst := cue.Build(load.Instances([]string{pkg}, &load.Config{
 			Dir:        dir,
 			ModuleRoot: dir,
-			Module:     "cuelang.org/go/encoding/gocode/testdata",
+			Module:     "cuelang.org/go/encoding/gocode/testdata@v0",
 		}))[0]
 		if err := inst.Err; err != nil {
 			log.Fatal(err)
