@@ -47,29 +47,21 @@ type loader struct {
 // - it includes valid modules for all of its dependencies
 // - it does not include any unnecessary dependencies.
 func CheckTidy(ctx context.Context, fsys fs.FS, modRoot string, reg Registry) error {
-	_, err := tidy(ctx, fsys, modRoot, reg, "", true)
+	_, err := tidy(ctx, fsys, modRoot, reg, true)
 	return err
 }
 
 // Tidy evaluates all the requirements of the given main module, using the given
 // registry to download requirements and returns a resolved and tidied module file.
-// If there's no language version in the module file and cueVers is non-empty
-// it will be used to populate the language version field.
-func Tidy(ctx context.Context, fsys fs.FS, modRoot string, reg Registry, cueVers string) (*modfile.File, error) {
-	return tidy(ctx, fsys, modRoot, reg, cueVers, false)
+func Tidy(ctx context.Context, fsys fs.FS, modRoot string, reg Registry) (*modfile.File, error) {
+	return tidy(ctx, fsys, modRoot, reg, false)
 }
 
-func tidy(ctx context.Context, fsys fs.FS, modRoot string, reg Registry, cueVers string, checkTidy bool) (*modfile.File, error) {
-	mainModuleVersion, mf, err := readModuleFile(ctx, fsys, modRoot)
+func tidy(ctx context.Context, fsys fs.FS, modRoot string, reg Registry, checkTidy bool) (*modfile.File, error) {
+	mainModuleVersion, mf, err := readModuleFile(fsys, modRoot)
 	if err != nil {
 		return nil, err
 	}
-	if checkTidy {
-		// This is the cheapest check, so do it first.
-		if mf.Language == nil || mf.Language.Version == "" {
-			return nil, fmt.Errorf("no language version found in cue.mod/module.cue")
-		}
-	}
 	// TODO check that module path is well formed etc
 	origRs := modrequirements.NewRequirements(mf.Module, reg, mf.DepVersions(), mf.DefaultMajorVersions())
 	rootPkgPaths, err := modimports.AllImports(modimports.AllModuleFiles(fsys, modRoot))
@@ -101,18 +93,38 @@ func tidy(ctx context.Context, fsys fs.FS, modRoot string, reg Registry, cueVers
 		return nil, fmt.Errorf("cannot tidy requirements: %v", err)
 	}
 	if ld.checkTidy && !equalRequirements(origRs, rs) {
-		// TODO be more specific in this error?
-		return nil, fmt.Errorf("module is not tidy")
+		// TODO: provide a reason, perhaps in structured form rather than a string
+		return nil, &ErrModuleNotTidy{}
 	}
-	return modfileFromRequirements(mf, rs, cueVers), nil
+	return modfileFromRequirements(mf, rs), nil
+}
+
+// ErrModuleNotTidy is returned by CheckTidy when a module is not tidy,
+// such as when there are missing or unnecessary dependencies listed.
+type ErrModuleNotTidy struct {
+	// Reason summarizes why the module is not tidy.
+	Reason string
+}
+
+func (e ErrModuleNotTidy) Error() string {
+	if e.Reason == "" {
+		return "module is not tidy"
+	}
+	return "module is not tidy: " + e.Reason
 }
 
 func equalRequirements(rs0, rs1 *modrequirements.Requirements) bool {
-	return slices.Equal(rs0.RootModules(), rs1.RootModules()) &&
+	// Note that rs1.RootModules may include the unversioned local module
+	// if the current module imports any packages under cue.mod/*/.
+	// In such a case we want to skip over the local module when comparing,
+	// just like modfileFromRequirements does when filling [modfile.File.Deps].
+	// Note that we clone the slice to not modify rs1's internal slice in-place.
+	rs1RootMods := slices.DeleteFunc(slices.Clone(rs1.RootModules()), module.Version.IsLocal)
+	return slices.Equal(rs0.RootModules(), rs1RootMods) &&
 		maps.Equal(rs0.DefaultMajorVersions(), rs1.DefaultMajorVersions())
 }
 
-func readModuleFile(ctx context.Context, fsys fs.FS, modRoot string) (module.Version, *modfile.File, error) {
+func readModuleFile(fsys fs.FS, modRoot string) (module.Version, *modfile.File, error) {
 	modFilePath := path.Join(modRoot, "cue.mod/module.cue")
 	data, err := fs.ReadFile(fsys, modFilePath)
 	if err != nil {
@@ -129,16 +141,16 @@ func readModuleFile(ctx context.Context, fsys fs.FS, modRoot string) (module.Ver
 	return mainModuleVersion, mf, nil
 }
 
-func modfileFromRequirements(old *modfile.File, rs *modrequirements.Requirements, cueVers string) *modfile.File {
+func modfileFromRequirements(old *modfile.File, rs *modrequirements.Requirements) *modfile.File {
+	// TODO it would be nice to have some way of automatically including new
+	// fields by default when they're added to modfile.File, but we don't
+	// want to just copy the entirety of old because that includes
+	// private fields too.
 	mf := &modfile.File{
 		Module:   old.Module,
 		Language: old.Language,
 		Deps:     make(map[string]*modfile.Dep),
-	}
-	if cueVers != "" && (mf.Language == nil || mf.Language.Version == "") {
-		mf.Language = &modfile.Language{
-			Version: cueVers,
-		}
+		Source:   old.Source,
 	}
 	defaults := rs.DefaultMajorVersions()
 	for _, v := range rs.RootModules() {
@@ -159,9 +171,18 @@ func (ld *loader) resolveDependencies(ctx context.Context, rootPkgPaths []string
 		pkgs := modpkgload.LoadPackages(ctx, ld.mainModule.Path(), ld.mainModuleLoc, rs, ld.registry, rootPkgPaths)
 		if ld.checkTidy {
 			for _, pkg := range pkgs.All() {
-				if err := pkg.Error(); err != nil {
-					return nil, nil, fmt.Errorf("module is not tidy: %v", err)
+				err := pkg.Error()
+				if err == nil {
+					continue
 				}
+				missingErr := new(modpkgload.ImportMissingError)
+				// "cannot find module providing package P" is confusing here,
+				// as checkTidy simply points out missing dependencies without fetching them.
+				if errors.As(err, &missingErr) {
+					err = &ErrModuleNotTidy{Reason: fmt.Sprintf(
+						"missing dependency providing package %s", missingErr.Path)}
+				}
+				return nil, nil, err
 			}
 			// All packages could be loaded OK so there are no new
 			// dependencies to be resolved and nothing to do.
